@@ -228,7 +228,7 @@ namespace conversions
 
 #if defined(__ANDROID__)
         template<class T>
-        inline std::string to_string(const T& t)
+        inline std::string to_string(const T t)
         {
             std::ostringstream os;
             os.imbue(std::locale::classic());
@@ -238,16 +238,16 @@ namespace conversions
 #endif
 
         template<class T>
-        inline utility::string_t to_string_t(T&& t)
+        inline utility::string_t to_string_t(const T t)
         {
 #ifdef _UTF16_STRINGS
             using std::to_wstring;
-            return to_wstring(std::forward<T>(t));
+            return to_wstring(t);
 #else
 #if !defined(__ANDROID__)
             using std::to_string;
 #endif
-            return to_string(std::forward<T>(t));
+            return to_string(t);
 #endif
         }
 
@@ -356,10 +356,16 @@ namespace details
     /// Our own implementation of alpha numeric instead of std::isalnum to avoid
     /// taking global lock for performance reasons.
     /// </summary>
-    inline bool __cdecl is_alnum(const unsigned char uch) noexcept
+    inline bool __cdecl is_alnum(const unsigned char uch) CPPREST_NOEXCEPT
     {   // test if uch is an alnum character
         // special casing char to avoid branches
-        static constexpr bool is_alnum_table[UCHAR_MAX + 1] =
+        static
+            #if !defined(_MSC_VER) || _MSC_VER >= 1900
+            constexpr
+            #else
+            const
+            #endif
+            bool is_alnum_table[UCHAR_MAX + 1] =
             {
             /*        X0 X1 X2 X3 X4 X5 X6 X7 X8 X9 XA XB XC XD XE XF */
             /* 0X */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -379,7 +385,7 @@ namespace details
     /// Our own implementation of alpha numeric instead of std::isalnum to avoid
     /// taking global lock for performance reasons.
     /// </summary>
-    inline bool __cdecl is_alnum(const char ch) noexcept
+    inline bool __cdecl is_alnum(const char ch) CPPREST_NOEXCEPT
     {
         return (is_alnum(static_cast<unsigned char>(ch)));
     }
@@ -389,7 +395,7 @@ namespace details
     /// taking global lock for performance reasons.
     /// </summary>
     template<class Elem>
-    inline bool __cdecl is_alnum(Elem ch) noexcept
+    inline bool __cdecl is_alnum(Elem ch) CPPREST_NOEXCEPT
     {
         // assumes 'x' == L'x' for the ASCII range
         typedef typename std::make_unsigned<Elem>::type UElem;
