@@ -51,7 +51,7 @@ namespace
         }
     };
 
-    constexpr to_lower_ch_impl to_lower_ch;
+    constexpr to_lower_ch_impl to_lower_ch{};
 
     struct eq_lower_ch_impl
     {
@@ -62,7 +62,7 @@ namespace
         }
     };
 
-    constexpr eq_lower_ch_impl eq_lower_ch;
+    constexpr eq_lower_ch_impl eq_lower_ch{};
 
     struct lt_lower_ch_impl
     {
@@ -73,7 +73,7 @@ namespace
         }
     };
 
-    constexpr lt_lower_ch_impl lt_lower_ch;
+    constexpr lt_lower_ch_impl lt_lower_ch{};
 }
 
 namespace utility
